### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.component import provideUtility
from interfaces import IAdverlet
from adverlet import Adverlet

def registerAdverlet(_context, name, description=None, default=None,
                     wysiwyg=True):

    adverlet = Adverlet()

    adverlet.__name__ = name
    adverlet.description = description
    adverlet.default = default
    adverlet.wysiwyg = wysiwyg
    adverlet.newlines = not wysiwyg

    provideUtility(adverlet, IAdverlet, name=name)



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################
""" Demo site
"""

__license__ = "GPL v.3"

from zope import event
from zope.app.folder import Folder
from zope.interface import Interface, implements
from zope.app.component.site import LocalSiteManager        
from ice.adverlet.interfaces import ISourceStorage, IFileStorage
from ice.adverlet.storage import SourceStorage, FileStorage

class ISite(Interface):
    """ Demo site """

class Site(Folder):
    implements(ISite)

    def __init__(self):
        super(Site, self).__init__()
        sm = LocalSiteManager(self)
        self.setSiteManager(sm)        
        sm = self.getSiteManager()
        event.notify(SiteCreatedEvent(self))

class ISiteCreatedEvent(Interface):
    """ event """

class SiteCreatedEvent:
    implements(ISiteCreatedEvent)
    
    def __init__(self, site):
        self.site = site                

def installs(e):
    site = e.site
    sm = site.getSiteManager()

    storage = SourceStorage()
    sm['adverlets_sources'] = storage
    sm.registerUtility(storage, ISourceStorage)
    
    storage = FileStorage()
    site['images'] = storage
    sm.registerUtility(storage, IFileStorage)
    



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################
""" Demo site, simple browser classes
"""

__license__ = "GPL v.3"

from zope.app.pagetemplate import ViewPageTemplateFile

class Frontpage(object):

    __call__ = ViewPageTemplateFile('layout.pt')
    render = ViewPageTemplateFile('frontpage.pt')

class Manage(object):

    __call__ = ViewPageTemplateFile('layout.pt')
    render = ViewPageTemplateFile('manage.pt')

class HeaderView(object):

    def __call__(self):
        return u'DEFAULT HEADER'

class MainView(object):

    def __call__(self):
        return u'DEFAULT MAIN'

class SidebarView(object):
    
    def __call__(self):
        return u'DEFAULT SIDEBAR'



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.component import adapter
from zope.interface import implementer
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.contentprovider.interfaces import IContentProvider
from interfaces import IManageUITemplate, IUndoTemplate

@implementer(IManageUITemplate)
@adapter(IContentProvider, IDefaultBrowserLayer)
def manageUITemplate(*argv):
    return ViewPageTemplateFile('manage.pt')

@implementer(IUndoTemplate)
@adapter(IContentProvider, IDefaultBrowserLayer)
def undoTemplate(*argv):
    return ViewPageTemplateFile('undo.pt')



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"
__docformat__ = 'restructuredtext'

from zope.interface import Interface
from zope.schema import TextLine, Text, Bool

class IAdverletDirective(Interface):
    """ Defines adverlet """

    name = TextLine(
        title = u'Name',
        description = u'Adverlets is looked up by the name',
        required = True)

    description = Text(
        title = u'Description',
        description = u'Useful description for content manager',
        required = False)

    default = TextLine(
        title = u'Default view name',
        description = u'Default view name',
        required = False)

    wysiwyg = Bool(
        title = u'Rich-text editor by default',
        description = u'Rich-text editor not always useful',
        required = False,
        default = True)



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.formlib import form
from zope.event import notify
from zope.interface import implements
from zope.component import getMultiAdapter
from zope.app.file.interfaces import IImage
from zope.cachedescriptors.property import Lazy
from zope.lifecycleevent import ObjectModifiedEvent
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.dublincore.interfaces import IZopeDublinCore
from zope.contentprovider.interfaces import IContentProvider
from zope.component import getUtilitiesFor, queryUtility, getUtility
from ice.adverlet.interfaces import IFileStorage, IImageWrapper
from ice.adverlet.interfaces import IAdverlet, ISourceStorage
from ice.adverlet.image import ImageWrapper
from interfaces import IManageUITemplate
from widget import RichTextWidget
from ice.adverlet.i18n import _

class ManageForm(form.FormBase):
    implements(IContentProvider)

    primary_actions = form.Actions()
    secondary_actions = form.Actions()
    preview = None
    info = None
    
    def __init__(self, context, request, view):
        self.__parent__ = view
        self.request = request
        self.context = self.getContext()
        self.default_css = self._sources.defaultCSS

        # customizable template
        self.template = getMultiAdapter(
            (self, self.request), IManageUITemplate,
            name='ice.adverlet.ManageTemplate')

        # edit adverlet
        if IAdverlet(self.context, None):
            self.title = _(u'Edit HTML')
            self.form_fields = form.Fields(IAdverlet)
            if IAdverlet(self.context).wysiwyg:
                self.form_fields['source'].custom_widget = RichTextWidget 
                self.form_fields = self.form_fields.omit('newlines')
                IAdverlet(self.context).newlines = False
            self.form_fields = self.form_fields.omit(
                '__name__', 'description', 'default')

        # edit settings
        elif ISourceStorage(self.context, None):
            self.title = _(u'Edit settings')
            self.form_fields = form.Fields(ISourceStorage)
            self.form_fields = self.form_fields.omit('defaultCSS')

        # upload file
        elif IImageWrapper(self.context, None):
            self.title = _(u'Upload File')
            self.form_fields = form.Fields(IImageWrapper)

        # for all forms
        if self.context:
            super(ManageForm, self).__init__(self.context, request)

    @Lazy
    def _sources(self):
        return getUtility(ISourceStorage)

    @Lazy
    def _files(self):
        return getUtility(IFileStorage)

    def getContext(self):
        request = self.request

        custom = request.get('custom')
        settings = request.get('settings')
        upload = request.get('images')

        # select type of form
        if custom:
            return queryUtility(IAdverlet, custom)
        elif settings:
            return self._sources
        elif upload:
            return ImageWrapper()
        else:
            return None

    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}     
        # context may be empty
        if self.context:
            self.widgets = form.setUpEditWidgets(
                self.form_fields, self.prefix, self.context, self.request,
                adapters=self.adapters, ignore_request=ignore_request
                )
        else:
            self.widgets = ()

    def update(self):
        request = self.request
        # for table with listing af all registered adverlets
        self.adverlets = [i[1] for i in getUtilitiesFor(IAdverlet)]

        # preview mode
        preview_name = request.get('preview')
        if preview_name:
            adv = queryUtility(IAdverlet, preview_name)
            self.adverlet_name = adv.__name__
            self.preview = adv and adv.source

        # delete image
        file_key = request.get('delete')
        if file_key:
            del self._files[file_key]
            self.redirect('?images=yes')

        # update form
        self.setUpWidgets()
        self.form_reset = False
        data = {}

        # for `cancel` action
        if form.handleSubmit(self.secondary_actions, data)[1]:
            self.redirect()

        # for `apply` action
        errors, action = form.handleSubmit(
            self.primary_actions, data, self.validate)

        if errors:
            self.status = u'There were errors'
            result = action.failure(data, errors)
        elif errors is not None:
            self.form_reset = True
            result = action.success(data)
        else:
            result = None

        self.form_result = result
        
        if action:
            custom = request.get('custom')
            
            self.redirect(
                (request.get('images') and '?images=yes') or
                (request.get('settings') and '?settings=yes') or
                (custom and '?custom=%s' % custom) or ''
                )

    @Lazy
    def actions(self):
        return list(self.primary_actions) + list(
            self.secondary_actions)

    @form.action(_(u'Apply'), primary_actions)
    def handle_apply_action(self, action, data):
        if form.applyChanges(self.context, self.form_fields,
                             data, self.adapters):
            notify(ObjectModifiedEvent(self.context))
            self.status = _(u'Updated')
        else:
            self.status = _(u'No changes')

    @form.action(_(u'Close'), secondary_actions)
    def handle_close_action(self, *argv):
        pass

    def redirect(self, query=''):
        self.request.response.redirect(
            str(self.request.URL) + query)

    @property
    def info(self):
        """ Some information """
        sources_size = sum(
            [len(source) for source in self._sources.sources.values()
             if source]
            )

        images_size = sum(
            [f.getSize() for f in self._files.values()
             if IImage(f, None)]
            )

        return {'sources_size':sources_size,
                'images_size':images_size}
 
    def getFiles(self):
        """ Listing of images """
        for id, ob in self._files.items():
            if IImage(ob, None):
                yield {'id':id, 'ob':ob, 'dc':IZopeDublinCore(ob)}

    def render(self):
        if self.form_result is None:
            if self.form_reset:
                self.resetForm()
                self.form_reset = False
            self.form_result = self.template(self)

        return self.form_result



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

import unittest
from zope.app.testing import functional

try:
    from z3c import sampledata
    sampledata = True
except ImportError:
    sampledata = False

functional.defineLayer('TestLayer', 'ftesting.zcml')

def setUp(test):
    pass

def tearDown(test):
    pass

def test_suite():
    suite = unittest.TestSuite()
    if not sampledata:
        return suite
    suites = (functional.FunctionalDocFileSuite(
            'ftests.txt', setUp=setUp, tearDown=tearDown
            ),)

    for s in suites:
        s.layer=TestLayer
        suite.addTest(s)

    return suite

    



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

import os.path
from zope.interface import implements
from zope.app.component.hooks import setSite
from zope.component import queryUtility, provideUtility
from z3c.sampledata.interfaces import ISampleDataPlugin
from ice.adverlet.interfaces import ISourceStorage, IFileStorage
from ice.adverlet.storage import SourceStorage, FileStorage

LOCAL_DIR = os.path.dirname(__file__)

class SampleSourceStorage(object):
    implements(ISampleDataPlugin)
    name = 'ice.adverlet.sample.sourcestorage'
    dependencies = []
    schema = None

    def generate(self, context, param={}, dataSource=None, seed=None):
        setSite(context)
        storage = queryUtility(ISourceStorage)
        if storage is None:
            storage = SourceStorage()
            sm = context.getSiteManager()
            sm['source_storage'] = storage
            provideUtility(storage, ISourceStorage)
        return storage

class SampleFileStorage(object):
    implements(ISampleDataPlugin)
    name = 'ice.adverlet.sample.filestorage'
    dependencies = []
    schema = None

    def generate(self, context, param={}, dataSource=None, seed=None):
        setSite(context)
        storage = queryUtility(IFileStorage)
        if storage is None:
            storage = FileStorage()
            sm = context.getSiteManager()
            sm['file_storage'] = storage
            provideUtility(storage, IFileStorage)
        return storage



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from StringIO import StringIO
from zope.interface import implements
from zope.tales.expressions import StringExpr
from zope.component import queryUtility, getMultiAdapter
from interfaces import ITALESAdverletExpression, IAdverlet

class TALESAdverletExpression(StringExpr):
    implements(ITALESAdverletExpression)
    
    def __call__(self, econtext):
        name = super(TALESAdverletExpression, self).__call__(econtext)
        adverlet = queryUtility(IAdverlet, name)

        if not adverlet:
            return None

        source = adverlet.source

        if source:
            return not adverlet.newlines and source or ''.join(
                ['%s <br />' % s for s in StringIO(source).readlines()])

        default = adverlet.default

        if not default:
            return None

        view = getMultiAdapter(
            (econtext.vars['context'], econtext.vars['request']),
            name=default)

        return view()



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.event import notify
from zope.interface import implements
from zope.component import queryUtility
from rwproperty import setproperty, getproperty
from zope.schema.fieldproperty import FieldProperty
from interfaces import IAdverlet, ISourceStorage
from events import SourceModifiedEvent

class Adverlet(object):
    """ See ice.adverlet.interfaces.IAdverlet """
    implements(IAdverlet)

    __name__ = __parent__ = None

    description = FieldProperty(IAdverlet['description'])
    default = FieldProperty(IAdverlet['default'])
    wysiwyg = FieldProperty(IAdverlet['wysiwyg'])
    newlines = FieldProperty(IAdverlet['newlines'])

    @setproperty
    def source(self, html):
        storage = queryUtility(ISourceStorage)

        if storage:
            storage.sources[self.__name__] = html
            notify(SourceModifiedEvent(self.__name__))

    @getproperty
    def source(self): 
        storage = queryUtility(ISourceStorage)

        if storage and self.__name__ in storage.sources:
            return storage.sources[self.__name__]

        return None



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from BTrees.OOBTree import OOBTree
from persistent import Persistent
from zope.interface import implements
from zope.app.container.contained import Contained
from zope.app.container.btree import BTreeContainer
from zope.schema.fieldproperty import FieldProperty
from interfaces import ISourceStorage, IFileStorage

class SourceStorage(Persistent, Contained):
    implements(ISourceStorage)

    mceplugins = FieldProperty(ISourceStorage['mceplugins'])
    defaultCSS = FieldProperty(ISourceStorage['defaultCSS'])

    def __init__(self):
        self.sources = OOBTree()

class FileStorage(BTreeContainer):
    implements(IFileStorage)



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.interface import Interface

class IManageUITemplate(Interface):
    """ Customizable template """

class IUndoTemplate(Interface):
    """ Customizable template """



### -*- coding: utf-8 -*- ############################################

__author__ = "Copyright (c) 2004 Zope Corporation and Contributors"
__license__ = "ZPL 2.1"

import os.path
from unittest import TestSuite
from doctest import DocFileSuite, NORMALIZE_WHITESPACE, ELLIPSIS
from zope.app.testing.setup import placefulSetUp, placefulTearDown
from zope.testing.doctestunit import DocFileSuite
from zope.security.management import getInteraction

class TestParticipation(object):
    principal = 'foobar'
    interaction = None

counter = 0
mtime_func = None

zptlogo = (
    'GIF89a\x10\x00\x10\x00\xd5\x00\x00\xff\xff\xff\xff\xff\xfe\xfc\xfd\xfd'
    '\xfa\xfb\xfc\xf7\xf9\xfa\xf5\xf8\xf9\xf3\xf6\xf8\xf2\xf5\xf7\xf0\xf4\xf6'
    '\xeb\xf1\xf3\xe5\xed\xef\xde\xe8\xeb\xdc\xe6\xea\xd9\xe4\xe8\xd7\xe2\xe6'
    '\xd2\xdf\xe3\xd0\xdd\xe3\xcd\xdc\xe1\xcb\xda\xdf\xc9\xd9\xdf\xc8\xd8\xdd'
    '\xc6\xd7\xdc\xc4\xd6\xdc\xc3\xd4\xda\xc2\xd3\xd9\xc1\xd3\xd9\xc0\xd2\xd9'
    '\xbd\xd1\xd8\xbd\xd0\xd7\xbc\xcf\xd7\xbb\xcf\xd6\xbb\xce\xd5\xb9\xcd\xd4'
    '\xb6\xcc\xd4\xb6\xcb\xd3\xb5\xcb\xd2\xb4\xca\xd1\xb2\xc8\xd0\xb1\xc7\xd0'
    '\xb0\xc7\xcf\xaf\xc6\xce\xae\xc4\xce\xad\xc4\xcd\xab\xc3\xcc\xa9\xc2\xcb'
    '\xa8\xc1\xca\xa6\xc0\xc9\xa4\xbe\xc8\xa2\xbd\xc7\xa0\xbb\xc5\x9e\xba\xc4'
    '\x9b\xbf\xcc\x98\xb6\xc1\x8d\xae\xbaFgs\x00\x00\x00\x00\x00\x00\x00\x00'
    '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    '\x00,\x00\x00\x00\x00\x10\x00\x10\x00\x00\x06z@\x80pH,\x12k\xc8$\xd2f\x04'
    '\xd4\x84\x01\x01\xe1\xf0d\x16\x9f\x80A\x01\x91\xc0ZmL\xb0\xcd\x00V\xd4'
    '\xc4a\x87z\xed\xb0-\x1a\xb3\xb8\x95\xbdf8\x1e\x11\xca,MoC$\x15\x18{'
    '\x006}m\x13\x16\x1a\x1f\x83\x85}6\x17\x1b $\x83\x00\x86\x19\x1d!%)\x8c'
    '\x866#\'+.\x8ca`\x1c`(,/1\x94B5\x19\x1e"&*-024\xacNq\xba\xbb\xb8h\xbeb'
    '\x00A\x00;'
    )

def setUp(test):
    placefulSetUp()

    from zope.app.pagetemplate.metaconfigure import registerType
    from ice.adverlet.tales import TALESAdverletExpression
    registerType('adverlet', TALESAdverletExpression)

    getInteraction().add(TestParticipation())

    # Make sure we are always reloading page template files ;-)
    global mtime_func
    mtime_func = os.path.getmtime
    def number(x):
        global counter
        counter += 1
        return counter
    os.path.getmtime = number


def tearDown(test):
    placefulTearDown()

def test_suite():
    return TestSuite(
        (DocFileSuite('README.txt',
                      package='ice.adverlet',
                      setUp=setUp, tearDown=tearDown,
                      optionflags=NORMALIZE_WHITESPACE | ELLIPSIS,
                      ),))



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.component import getUtilitiesFor
from zope.schema.vocabulary import SimpleVocabulary

def tinyMcePluginsVocabulary(context):
    plugins = [
        'style', 'layer', 'table', 'save', 'advhr', 'advimage',
        'advlink', 'emotions' , 'iespell', 'insertdatetime',
        'preview', 'zoom', 'media', 'searchreplace', 'print',
        'contextmenu', 'paste', 'directionality', 'fullscreen',
        'noneditable', 'visualchars', 'nonbreaking', 'xhtmlxtras'
        ]

    return SimpleVocabulary.fromValues(plugins)



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.event import notify
from zope.interface import implements
from zope.component import getUtility
from rwproperty import setproperty, getproperty
from zope.cachedescriptors.property import Lazy
from zope.lifecycleevent import ObjectCreatedEvent
from zope.app.file.image import Image, getImageInfo
from zope.dublincore.interfaces import IZopeDublinCore
from zope.app.container.interfaces import INameChooser
from interfaces import IFileStorage, IImageWrapper
from i18n import _

class ImageWrapper(object):
    implements(IImageWrapper)

    def __init__(self):
        self.contentType = None

    @Lazy
    def _storage(self):
        return getUtility(IFileStorage)

    @setproperty
    def data(self, data):
        image = Image(data)  
        notify(ObjectCreatedEvent(image))
        id = INameChooser(self._storage).chooseName(None, image)
        self.image = self._storage[id] = image
        self.contentType = getImageInfo(data)[0]
    
    @setproperty
    def description(self, text):
        IZopeDublinCore(self.image).title = text

    @getproperty
    def data(self):
        return None

    @getproperty
    def description(self):
        return None

    def getSize(self):
        return self.image.getSize()
    
    def getImageSize(self):
        return self.image.getImageSize()



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"
__docformat__ = 'restructuredtext'

from zope.interface import Interface, Attribute
from zope.tales.interfaces import ITALESExpression
from zope.schema import Text, TextLine, List, Choice, Bool
from zope.location.interfaces import ILocation
from zope.app.file.interfaces import IImage
from i18n import _

class IAdverlet(ILocation):
    """ Adverlet """

    description = Text(title=_(u'Description'), required=False)

    default = TextLine(title=_(u'Default view name'), required=False)

    source = Text(title=_(u'HTML Source'), required=False)

    wysiwyg = Bool(title=_(u'Rich-text editor'), default=True)

    newlines = Bool(title=_(u'Render newlines'), default=False)

class ISourceStorage(Interface):
    """ Storage for HTML sources """

    sources = Attribute('HTML sources')

    mceplugins = List(
        title=_(u'TinyMCE Plugins'),
        default=[],
        value_type=Choice(vocabulary='ice.adverlet.mceplugins'))
    
    defaultCSS = Bool(
        title=_(u'Include default css-styles for management UI'),
        default=True)

class IFileStorage(Interface):
    """ Files storage """

class IImageWrapper(IImage):
    """ Image wrapper """

    description = TextLine(title=_(u'Description'))

class ISourceModifiedEvent(Interface):
    """ Event """

class ITALESAdverletExpression(ITALESExpression):
    """ Returns a HTML content of the adverlet.
    To call a adverlet in a view use the follow syntax
    in a page template::

      <tal:block content="structure adverlet:adverlet_name" />

    Thus, adverlet is looked up only by the name.
    """



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from z3c.widget.tiny.widget import TinyWidget
from zope.component import getUtility
from ice.adverlet.interfaces import ISourceStorage

class RichTextWidget(TinyWidget):

    height = 25
    mce_theme = "advanced"
    mce_theme_advanced_toolbar_location = "top"
    mce_theme_advanced_toolbar_align="left"
    mce_theme_advanced_statusbar_location = "bottom"
    width = "100%"
    mce_entity_encoding = "raw"
    mce_convert_newlines_to_brs = "false"
    mce_relative_urls = "false"
    mce_theme_advanced_buttons1 = "bold,italic,underline,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,|,bullist,numlist,|,outdent,indent,|,formatselect,fontselect,fontsizeselect"
    mce_theme_advanced_buttons2 = "cut,copy,paste,pastetext,pasteword,|,undo,redo,|,link,unlink,anchor,image,cleanup,code,|,insertdate,inserttime,|,forecolor,backcolor"
    mce_theme_advanced_buttons3 = "tablecontrols,|,hr,removeformat,visualaid,|,sub,sup,|,charmap,iespell,media,advhr,|,ltr,rtl,|,preview,fullscreen"
    mce_extended_valid_elements = "a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],font[face|size|color|style],span[class|align|style]"

    @property
    def mce_plugins(self):
        tool = getUtility(ISourceStorage)
        return ','.join(tool.mceplugins)



import sys, os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version='0.2.0'

setup(
    name='ice.adverlet',
    version='0.2.0',
    author='Ilshad Habibullin',
    author_email = 'astoon.net at gmail.com',
    url='http://launchpad.net/ice.adverlet',
    description = 'Simple way to edit any HTML snippet',
    long_description = (
        read('src/ice/adverlet/README.txt')
        + '\n\n' +
        read('src/ice/adverlet/demo/README.txt')
        + '\n\n' +
        read('CHANGES.txt')
        ),
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Framework :: Zope3'],
    license = 'GPL v.3',
   
    packages=find_packages('src'),
    namespace_packages=['ice',],
    package_dir={'':'src'},

    extras_require = dict(
        test = ['zope.app.testing',
                'zope.app.twisted',
                'zope.app.cache',
                'zope.app.catalog',
                'zope.app.securitypolicy',
                'zope.contentprovider',
                'zope.testbrowser',
                'z3c.autoinclude',
                'z3c.sampledata'
                ]
        ),

    install_requires=['setuptools',
                      'rwproperty',
                      'z3c.autoinclude',
                      'zope.contentprovider',
                      'zope.app.undo',
                      'zc.resourcelibrary',
                      'z3c.widget',
                      # demo app
                      'zope.app.zcmlfiles',
                      'zope.app.twisted',
                      'zope.app.securitypolicy',
                      'zope.app.catalog',
                      'zope.app.apidoc',
                      'zope.app.preference'
                      ],

    dependency_links = ['http://download.zope.org/distribution'], 
    include_package_data=True,
    zip_safe=False,
    )



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope.interface import implements
from zope.cachedescriptors.property import Lazy
from zope.app.undo.interfaces import IUndoManager
from zope.component import getUtility, getMultiAdapter
from zope.contentprovider.interfaces import IContentProvider
from zope.app.security.interfaces import IAuthentication
from zope.app.pagetemplate import ViewPageTemplateFile
from ice.adverlet.interfaces import ISourceStorage
from interfaces import IUndoTemplate
from ice.adverlet.i18n import _

class Undo(object):
    implements(IContentProvider)
    
    def __init__(self, context, request, view):
        self.__parent__ = view
        self.request = request
        self.context = context

        # customizable template
        self.template = getMultiAdapter(
            (self, self.request), IUndoTemplate,
            name='ice.adverlet.UndoTemplate')

    @Lazy
    def _manager(self):
        return getUtility(IUndoManager)

    def update(self):
        request = self.request
        self.transactions = self._manager.getPrincipalTransactions(
            request.principal, self.context, 0, -10)

        id = request.get('id')
        if id:
            self._manager.undoPrincipalTransactions(
                request.principal, ids=[id,])

            request.response.redirect(str(request.URL) + '?undo=yes')

    def render(self):
        return self.template(self)



### -*- coding: utf-8 -*- ############################################
### Author: Ilshad Habibullin, 2008 <astoon.net@gmail.com> ###########
######################################################################

__license__ = "GPL v.3"

from zope import event
from zope.interface import implements
from interfaces import ISourceModifiedEvent

class SourceModifiedEvent:
    implements(ISourceModifiedEvent)
    
    def __init__(self, name):
        self.name = name
